#include <fstream>
#include <iostream>
#include <queue>
#include <vector>
#include <functional>
#define INFINITE 1000000000
#define MAX_NODES 100001

using namespace std;

ifstream in("input.txt");
ofstream out("output.txt");

int edges, nodes, start, finish, cost, restaurant;
vector<pair<int, int>> city_graph[MAX_NODES];
int scooters, orders, order, order_time;
priority_queue<int, vector<int>, greater<int>> order_queue;

// minimum time/cost for getting from the restaurant to a destination location
int min_cost[MAX_NODES];

class MinCostComparator
{
public:
    bool operator () (const pair<int, int>& lhs, const pair<int, int>& rhs)
    {
        return lhs.second > rhs.second ||
            (lhs.second == rhs.second && lhs.first > rhs.first);
    }
};

void dijkstra()
{
    priority_queue<pair<int, int>, vector<pair<int, int>>, MinCostComparator> min_heap;
    int top_min;
    for (int location = 1; location <= nodes; ++location)
    {
        min_cost[location] = INFINITE;
    }
    min_cost[restaurant] = 0;
    min_heap.push(make_pair(restaurant, 0));
    while (!min_heap.empty())
    {
        top_min = min_heap.top().first;
        min_heap.pop();
        for (unsigned location = 0; location < city_graph[top_min].size(); ++location)
        {
            if (min_cost[city_graph[top_min][location].first] > min_cost[top_min] + city_graph[top_min][location].second)
            {
                min_cost[city_graph[top_min][location].first] = min_cost[top_min] + city_graph[top_min][location].second;
                min_heap.push(make_pair(city_graph[top_min][location].first, min_cost[city_graph[top_min][location].first]));
            }
        }
    }
}

void solve()
{
    in >> nodes >> edges;
    while (edges--)
    {
        in >> start >> finish >> cost;
        city_graph[start].push_back(make_pair(finish, cost));
        // add (destination, source) edge too, graph is undirected
        city_graph[finish].push_back(make_pair(start, cost));
    }
    in >> restaurant;
    // find lowest cost for a path from the restaurant to all map locations using Dijkstra
    dijkstra();

    in >> scooters >> orders;
    int min_time = 0;
    while (orders--)
    {
        in >> order;
        // time is doubled as the scooter needs to get back to the restaurant for the next order
        order_time = 2 * min_cost[order];
        if (scooters <= 0)
        {
            // after assigning all scooters, take the scooter that will return first from its order
            // and give it the next order; add the next order time to the scooter's completed time
            order_time += order_queue.top();
            order_queue.pop();
        }
        else
        {
            // just assign the first orders to all available scooters and decrement scooter count
            scooters--;
        }
        // the maximum time it will take for all scooters to complete the orders is computed as
        // the maximum of the time needed for each scooter to arrive back at the restaurant
        min_time = max(min_time, order_time);
        order_queue.push(order_time);
    }
    out << min_time;
}

int main()
{
    solve();
    return 0;
}
